gmail = ""
gmail_pw = ""

fred_key = ''

binance_key = ''
binance_secret = ''

census_bureau_key = ''

quandl_key = ''

twitter_key = ''
twitter_secret_key = ''
twitter_token = ''
twitter_token_secret = ''

eia_key = ''

newsapi_key = ''
